@@ -1,11 +1,15 @@
 #ifndef NSTD_CORE_CORE_H_INCLUDED
 #define NSTD_CORE_CORE_H_INCLUDED
+#include "../nstd.h"
 #include "def.h"
-#ifdef __cplusplus
+#ifdef NSTDCPP
 extern "C"
 {
 #endif
 
+/// A null pointer value.
+#define NSTD_CORE_NULL 0
+
 /// Terminates the program in an abnormal fashion.
 NSTDAPI void nstd_core_abort();
 
@@ -14,7 +18,7 @@ NSTDAPI void nstd_core_abort();
 ///     `const NSTDBool b` - The boolean.
 NSTDAPI void nstd_core_assert(const NSTDBool b);
 
-#ifdef __cplusplus
+#ifdef NSTDCPP
 }
 #endif
 #endif
