@@ -1,7 +1,8 @@
 #ifndef NSTD_CORE_POINTER_H_INCLUDED
 #define NSTD_CORE_POINTER_H_INCLUDED
+#include "../nstd.h"
 #include "def.h"
-#ifdef __cplusplus
+#ifdef NSTDCPP
 extern "C"
 {
 #endif
@@ -20,15 +21,15 @@ typedef struct
 ///     `const NSTDAny obj` - The object to reference.
 ///     `const NSTDUSize size` - The size in bytes of `obj`.
 /// Returns: `NSTDPointer ptr` - The pointer type.
-NSTDAPI NSTDPointer nstd_core_pointer_new(NSTDAny obj, const NSTDUSize size);
+NSTDAPI NSTDPointer nstd_core_pointer_new(const NSTDAny obj, const NSTDUSize size);
 
 /// Overwrites the current referenced object's data with `obj`.
 /// Parameters:
 ///     `NSTDPointer *const ptr` - The pointer.
-///     `NSTDAnyConst obj` - The object to overwrite with.
-NSTDAPI void nstd_core_pointer_write(NSTDPointer *const ptr, NSTDAnyConst obj);
+///     `const NSTDAnyConst obj` - The object to overwrite with.
+NSTDAPI void nstd_core_pointer_write(NSTDPointer *const ptr, const NSTDAnyConst obj);
 
-#ifdef __cplusplus
+#ifdef NSTDCPP
 }
 #endif
 #endif
