@@ -1,12 +1,7 @@
 use crate::{
-    core::slice::NSTDSlice,
+    core::{def::NSTDErrorCode, slice::NSTDSlice},
     gui::{NSTDWindow, NSTDWindowSize},
-};
-use futures::executor;
-use std::{
-    ffi::CString,
-    os::raw::{c_char, c_int},
-    ptr,
+    string::NSTDString,
 };
 use wgpu::{util::*, *};
 
@@ -44,23 +39,30 @@ pub type NSTDGLBuffer = *mut Buffer;
 /// Represents a GL state.
 #[repr(C)]
 pub struct NSTDGLState {
+    /// The surface to draw on.
     pub surface: NSTDGLSurface,
+    /// The surface configuration.
     pub config: NSTDGLSurfaceConfiguration,
+    /// A handle to the drawing device.
     pub device_handle: NSTDGLDeviceHandle,
+    /// The drawing device.
     pub device: NSTDGLDevice,
+    /// The device's command queue.
     pub queue: NSTDGLQueue,
+    /// The size of the window.
     pub size: NSTDWindowSize,
+    /// The window's clear color.
     pub clear_color: NSTDGLColor,
 }
 impl Default for NSTDGLState {
     #[inline]
     fn default() -> Self {
         Self {
-            surface: ptr::null_mut(),
-            config: ptr::null_mut(),
-            device_handle: ptr::null_mut(),
-            device: ptr::null_mut(),
-            queue: ptr::null_mut(),
+            surface: std::ptr::null_mut(),
+            config: std::ptr::null_mut(),
+            device_handle: std::ptr::null_mut(),
+            device: std::ptr::null_mut(),
+            queue: std::ptr::null_mut(),
             size: NSTDWindowSize::new(0, 0),
             clear_color: NSTDGLColor::default(),
         }
@@ -71,12 +73,19 @@ impl Default for NSTDGLState {
 #[repr(C)]
 #[allow(non_camel_case_types)]
 pub enum NSTDGLBackend {
+    /// An unknown graphics backend.
     NSTD_GL_BACKEND_UNKNOWN,
+    /// Vulkan.
     NSTD_GL_BACKEND_VULKAN,
+    /// Metal.
     NSTD_GL_BACKEND_METAL,
+    /// Direct3D 12.
     NSTD_GL_BACKEND_DX12,
+    /// Direct3D 11.
     NSTD_GL_BACKEND_DX11,
+    /// OpenGL.
     NSTD_GL_BACKEND_GL,
+    /// Web based GPU.
     NSTD_GL_BACKEND_WEBGPU,
 }
 impl Into<Backends> for NSTDGLBackend {
@@ -112,10 +121,15 @@ impl From<Backend> for NSTDGLBackend {
 #[repr(C)]
 #[allow(non_camel_case_types)]
 pub enum NSTDGLDeviceType {
+    /// An unknown device type.
     NSTD_GL_DEVICE_TYPE_UNKNOWN,
+    /// `wgpu`'s integrated GPU.
     NSTD_GL_DEVICE_TYPE_INTEGRATED_GPU,
+    /// A physical GPU.
     NSTD_GL_DEVICE_TYPE_DISCRETE_GPU,
+    /// A virtual/hosted GPU.
     NSTD_GL_DEVICE_TYPE_VIRTUAL_GPU,
+    /// CPU/Software rendering.
     NSTD_GL_DEVICE_TYPE_CPU,
 }
 impl From<DeviceType> for NSTDGLDeviceType {
@@ -134,10 +148,15 @@ impl From<DeviceType> for NSTDGLDeviceType {
 /// Contains information on a device.
 #[repr(C)]
 pub struct NSTDGLDeviceInfo {
-    pub name: *mut c_char,
+    /// The name of the drawing device.
+    pub name: NSTDString,
+    /// The device's vendor.
     pub vendor: usize,
+    /// The ID of the device adapter.
     pub device: usize,
+    /// The type of drawing device.
     pub device_type: NSTDGLDeviceType,
+    /// The drawing backend in use.
     pub backend: NSTDGLBackend,
 }
 
@@ -145,8 +164,12 @@ pub struct NSTDGLDeviceInfo {
 #[repr(C)]
 #[allow(non_camel_case_types)]
 pub enum NSTDGLPresentationMode {
+    /// `wgpu`'s presentation engine will request drawing immediately.
     NSTD_GL_PRESENTATION_MODE_IMMEDIATE,
+    /// Waits for the vertical blanking period, but frames are submitted immediately.
     NSTD_GL_PRESENTATION_MODE_MAILBOX,
+    /// Waits for the vertical blanking period, and frames are
+    /// submitted with the monitor's referesh rate.
     NSTD_GL_PRESENTATION_MODE_FIFO,
 }
 impl Into<PresentMode> for NSTDGLPresentationMode {
@@ -164,8 +187,11 @@ impl Into<PresentMode> for NSTDGLPresentationMode {
 #[repr(C)]
 #[allow(non_camel_case_types)]
 pub enum NSTDGLPowerPreference {
+    /// Use the default power preference.
     NSTD_GL_POWER_PREFERENCE_DEFAULT,
+    /// Use low GPU power.
     NSTD_GL_POWER_PREFERENCE_LOW,
+    /// Use high GPU power.
     NSTD_GL_POWER_PREFERENCE_HIGH,
 }
 impl Into<PowerPreference> for NSTDGLPowerPreference {
@@ -183,8 +209,11 @@ impl Into<PowerPreference> for NSTDGLPowerPreference {
 /// For `backend`, `NSTD_GL_BACKEND_UNKNOWN` will pick a default backend to use.
 #[repr(C)]
 pub struct NSTDGLStateDescriptor {
+    /// The graphics backend to use.
     pub backend: NSTDGLBackend,
+    /// The amount of GPU power to be used.
     pub power_preference: NSTDGLPowerPreference,
+    /// The way frames will be presented to the display.
     pub presentation_mode: NSTDGLPresentationMode,
 }
 
@@ -193,39 +222,73 @@ pub struct NSTDGLStateDescriptor {
 #[derive(Clone, Copy)]
 #[allow(non_camel_case_types)]
 pub enum NSTDGLVertexFormat {
+    /// Two `UINT8`s.
     NSTD_GL_VERTEX_FORMAT_UINT8X2,
+    /// Four `UINT8`s.
     NSTD_GL_VERTEX_FORMAT_UINT8X4,
+    /// Two `INT8`s.
     NSTD_GL_VERTEX_FORMAT_INT8X2,
+    /// Four `INT8`s.
     NSTD_GL_VERTEX_FORMAT_INT8X4,
+    /// Two `UNORM8`s.
     NSTD_GL_VERTEX_FORMAT_UNORM8X2,
+    /// Four `UNORM8`s.
     NSTD_GL_VERTEX_FORMAT_UNORM8X4,
+    /// Two `NORM8`s.
     NSTD_GL_VERTEX_FORMAT_NORM8X2,
+    /// Four `NORM8`s.
     NSTD_GL_VERTEX_FORMAT_NORM8X4,
+    /// Two `UINT16`s.
     NSTD_GL_VERTEX_FORMAT_UINT16X2,
+    /// Four `UINT16`s.
     NSTD_GL_VERTEX_FORMAT_UINT16X4,
+    /// Two `INT16`s.
     NSTD_GL_VERTEX_FORMAT_INT16X2,
+    /// Four `INT16`s.
     NSTD_GL_VERTEX_FORMAT_INT16X4,
+    /// Two `UNORM16`s.
     NSTD_GL_VERTEX_FORMAT_UNORM16X2,
+    /// Four `UNORM16`s.
     NSTD_GL_VERTEX_FORMAT_UNORM16X4,
+    /// Two `NORM16`s.
     NSTD_GL_VERTEX_FORMAT_NORM16X2,
+    /// Four `NORM16`s.
     NSTD_GL_VERTEX_FORMAT_NORM16X4,
+    /// Two `FLOAT16`s.
     NSTD_GL_VERTEX_FORMAT_FLOAT16X2,
+    /// Four `FLOAT16`s.
     NSTD_GL_VERTEX_FORMAT_FLOAT16X4,
+    /// One `FLOAT32`.
     NSTD_GL_VERTEX_FORMAT_FLOAT32,
+    /// Two `FLOAT32`s.
     NSTD_GL_VERTEX_FORMAT_FLOAT32X2,
+    /// Three `FLOAT32`s.
     NSTD_GL_VERTEX_FORMAT_FLOAT32X3,
+    /// Four `FLOAT32`s.
     NSTD_GL_VERTEX_FORMAT_FLOAT32X4,
+    /// One `UINT32`.
     NSTD_GL_VERTEX_FORMAT_UINT32,
+    /// Two `UINT32`s.
     NSTD_GL_VERTEX_FORMAT_UINT32X2,
+    /// Three `UINT32`s.
     NSTD_GL_VERTEX_FORMAT_UINT32X3,
+    /// Four `UINT32`s.
     NSTD_GL_VERTEX_FORMAT_UINT32X4,
+    /// One `INT32`.
     NSTD_GL_VERTEX_FORMAT_INT32,
+    /// Two `INT32`s.
     NSTD_GL_VERTEX_FORMAT_INT32X2,
+    /// Three `INT32`s.
     NSTD_GL_VERTEX_FORMAT_INT32X3,
+    /// Four `INT32`s.
     NSTD_GL_VERTEX_FORMAT_INT32X4,
+    /// One `FLOAT64`.
     NSTD_GL_VERTEX_FORMAT_FLOAT64,
+    /// Two `FLOAT64`s.
     NSTD_GL_VERTEX_FORMAT_FLOAT64X2,
+    /// Three `FLOAT64`s.
     NSTD_GL_VERTEX_FORMAT_FLOAT64X3,
+    /// Four `FLOAT64`s.
     NSTD_GL_VERTEX_FORMAT_FLOAT64X4,
 }
 impl Into<VertexFormat> for NSTDGLVertexFormat {
@@ -275,7 +338,9 @@ impl Into<VertexFormat> for NSTDGLVertexFormat {
 #[derive(Clone, Copy)]
 #[allow(non_camel_case_types)]
 pub enum NSTDGLIndexFormat {
+    /// `NSTDUInt16`.
     NSTD_GL_INDEX_FORMAT_UINT16,
+    /// `NSTDUInt32`.
     NSTD_GL_INDEX_FORMAT_UINT32,
 }
 impl Into<IndexFormat> for NSTDGLIndexFormat {
@@ -294,8 +359,11 @@ impl Into<IndexFormat> for NSTDGLIndexFormat {
 #[repr(C)]
 #[derive(Clone, Copy)]
 pub struct NSTDGLVertexAttribute {
+    /// The vertex format to be used.
     pub format: NSTDGLVertexFormat,
+    /// The offset in bytes from the start of the input.
     pub offset: u64,
+    /// The location for this input.
     pub location: u32,
 }
 impl Into<VertexAttribute> for NSTDGLVertexAttribute {
@@ -314,7 +382,9 @@ impl Into<VertexAttribute> for NSTDGLVertexAttribute {
 #[derive(Clone, Copy)]
 #[allow(non_camel_case_types)]
 pub enum NSTDGLVertexStepMode {
+    /// Vertex data is advanced each vertex.
     NSTD_GL_VERTEX_STEP_MODE_VERTEX,
+    /// Vertex data is advanced each instance.
     NSTD_GL_VERTEX_STEP_MODE_INSTANCE,
 }
 impl Into<VertexStepMode> for NSTDGLVertexStepMode {
@@ -331,8 +401,11 @@ impl Into<VertexStepMode> for NSTDGLVertexStepMode {
 /// `attributes` - `&mut [NSTDGLVertexAttribute]`.
 #[repr(C)]
 pub struct NSTDGLVertexBufferLayout {
+    /// The number of bytes between each element.
     pub stride: u64,
+    /// Determines how often the vertex data is advanced.
     pub step_mode: NSTDGLVertexStepMode,
+    /// A slice of `NSTDGLVertexAttribute`s.
     pub attributes: NSTDSlice,
 }
 impl<'a> Into<VertexBufferLayout<'a>> for NSTDGLVertexBufferLayout {
@@ -353,7 +426,7 @@ impl<'a> Into<VertexBufferLayout<'a>> for NSTDGLVertexBufferLayout {
 
 /// Creates a new GL state.
 /// Parameters:
-///     `NSTDWindow window` - The window in which the GL state will live in.
+///     `const NSTDWindow window` - The window in which the GL state will live in.
 ///     `const NSTDGLStateDescriptor descriptor` - Configures the state.
 /// Returns: `NSTDGLState state` - The new GL state.
 #[cfg_attr(feature = "clib", no_mangle)]
@@ -363,19 +436,21 @@ pub unsafe extern "C" fn nstd_gl_state_new(
 ) -> NSTDGLState {
     let instance = Instance::new(descriptor.backend.into());
     let surface = instance.create_surface(&*window);
-    let adapter = match executor::block_on(instance.request_adapter(&RequestAdapterOptions {
-        power_preference: descriptor.power_preference.into(),
-        compatible_surface: Some(&surface),
-        force_fallback_adapter: false,
-    })) {
-        Some(adapter) => adapter,
-        _ => return NSTDGLState::default(),
-    };
-    let (device, queue) =
-        match executor::block_on(adapter.request_device(&DeviceDescriptor::default(), None)) {
-            Ok((device, queue)) => (device, queue),
+    let adapter =
+        match futures::executor::block_on(instance.request_adapter(&RequestAdapterOptions {
+            power_preference: descriptor.power_preference.into(),
+            compatible_surface: Some(&surface),
+            force_fallback_adapter: false,
+        })) {
+            Some(adapter) => adapter,
             _ => return NSTDGLState::default(),
         };
+    let (device, queue) = match futures::executor::block_on(
+        adapter.request_device(&DeviceDescriptor::default(), None),
+    ) {
+        Ok((device, queue)) => (device, queue),
+        _ => return NSTDGLState::default(),
+    };
     let size = crate::gui::nstd_gui_window_get_client_size(window);
     let config = SurfaceConfiguration {
         usage: TextureUsages::RENDER_ATTACHMENT,
@@ -403,12 +478,12 @@ pub unsafe extern "C" fn nstd_gl_state_new(
 /// Parameters:
 ///     `const NSTDGLState *const state` - The GL state.
 ///     `void(*callback)(NSTDGLRenderPass)` - Manipulates the render pass.
-/// Returns: `int errc` - Nonzero on error.
+/// Returns: `NSTDErrorCode errc` - Nonzero on error.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_gl_state_render(
     state: &NSTDGLState,
     callback: extern "C" fn(NSTDGLRenderPass),
-) -> c_int {
+) -> NSTDErrorCode {
     let output = match (*state.surface).get_current_texture() {
         Ok(output) => output,
         _ => return 1,
@@ -462,16 +537,16 @@ pub unsafe extern "C" fn nstd_gl_state_free(state: &mut NSTDGLState) {
     Box::from_raw(state.device_handle);
     Box::from_raw(state.device);
     Box::from_raw(state.queue);
-    state.surface = ptr::null_mut();
-    state.config = ptr::null_mut();
-    state.device_handle = ptr::null_mut();
-    state.device = ptr::null_mut();
-    state.queue = ptr::null_mut();
+    state.surface = std::ptr::null_mut();
+    state.config = std::ptr::null_mut();
+    state.device_handle = std::ptr::null_mut();
+    state.device = std::ptr::null_mut();
+    state.queue = std::ptr::null_mut();
 }
 
 /// Retrieves info on a device.
 /// Parameters:
-///     `NSTDGLDeviceHandle device_handle` - Handle to a device.
+///     `const NSTDGLDeviceHandle device_handle` - Handle to a device.
 /// Returns: `NSTDGLDeviceInfo device_info` - Contains information about a device.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_gl_device_handle_get_info(
@@ -479,11 +554,7 @@ pub unsafe extern "C" fn nstd_gl_device_handle_get_info(
 ) -> NSTDGLDeviceInfo {
     let info = (*device_handle).get_info();
     NSTDGLDeviceInfo {
-        name: {
-            let mut bytes = info.name.into_bytes();
-            bytes.push(0);
-            CString::from_vec_unchecked(bytes).into_raw()
-        },
+        name: NSTDString::from(info.name.as_bytes()),
         vendor: info.vendor,
         device: info.device,
         device_type: NSTDGLDeviceType::from(info.device_type),
@@ -494,7 +565,7 @@ pub unsafe extern "C" fn nstd_gl_device_handle_get_info(
 /// Creates a new shader module.
 /// Parameters:
 ///     `const NSTDSlice *const data` - Raw spirv data.
-///     `NSTDGLDevice device` - The device to create the shader module on.
+///     `const NSTDGLDevice device` - The device to create the shader module on.
 /// Returns: `NSTDGLShaderModule shader` - The new shader module.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_gl_shader_module_new(
@@ -512,21 +583,21 @@ pub unsafe extern "C" fn nstd_gl_shader_module_new(
 
 /// Frees a shader module.
 /// Parameters:
-///     `NSTDGLShaderModule *shader` - Pointer to a shader module.
+///     `NSTDGLShaderModule *const shader` - Pointer to a shader module.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_gl_shader_module_free(shader: &mut NSTDGLShaderModule) {
     Box::from_raw(*shader);
-    *shader = ptr::null_mut();
+    *shader = std::ptr::null_mut();
 }
 
 /// Creates a new render pipeline from a vertex and fragment shader.
 /// Parameters:
-///     `NSTDGLShaderModule vert` - The vertex shader module.
-///     `NSTDGLShaderModule frag` - The fragment shader module.
+///     `const NSTDGLShaderModule vert` - The vertex shader module.
+///     `const NSTDGLShaderModule frag` - The fragment shader module.
 ///     `const NSTDSlice *const buffers` - A slice of `NSTDGLVertexBufferLayout`s.
-///     `NSTDGLDevice device` - The device to create the render pipeline on.
-///     `NSTDGLSurfaceConfiguration config` - The surface configuration.
+///     `const NSTDGLDevice device` - The device to create the render pipeline on.
+///     `const NSTDGLSurfaceConfiguration config` - The surface configuration.
 /// Returns: `NSTDGLRenderPipeline pipeline` - The new render pipeline.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_gl_render_pipeline_new(
@@ -597,18 +668,18 @@ pub unsafe extern "C" fn nstd_gl_render_pipeline_new(
 
 /// Frees a render pipeline.
 /// Parameters:
-///     `NSTDGLRenderPipeline *pipeline` - Pointer to a render pipeline.
+///     `NSTDGLRenderPipeline *const pipeline` - Pointer to a render pipeline.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_gl_render_pipeline_free(pipeline: &mut NSTDGLRenderPipeline) {
     Box::from_raw(*pipeline);
-    *pipeline = ptr::null_mut();
+    *pipeline = std::ptr::null_mut();
 }
 
 /// Sets a render pipeline for a render pass.
 /// Parameters:
-///     `NSTDGLRenderPass render_pass` - The render pass.
-///     `NSTDGLRenderPipeline pipeline` - The render pipeline.
+///     `const NSTDGLRenderPass render_pass` - The render pass.
+///     `const NSTDGLRenderPipeline pipeline` - The render pipeline.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_gl_render_pass_set_pipeline(
@@ -620,8 +691,8 @@ pub unsafe extern "C" fn nstd_gl_render_pass_set_pipeline(
 
 /// Sets a render pass' vertex buffer.
 /// Parameters:
-///     `NSTDGLRenderPass render_pass` - The render pass.
-///     `NSTDGLBuffer buffer` - The GPU vertex buffer.
+///     `const NSTDGLRenderPass render_pass` - The render pass.
+///     `const NSTDGLBuffer buffer` - The GPU vertex buffer.
 ///     `const NSTDUInt32 slot` - The buffer slot (the index of the buffer layout).
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
@@ -635,9 +706,9 @@ pub unsafe extern "C" fn nstd_gl_render_pass_set_vertex_buffer(
 
 /// Sets a render pass' index buffer.
 /// Parameters:
-///     `NSTDGLRenderPass render_pass` - The render pass.
-///     `NSTDGLBuffer buffer` - The GPU index buffer.
-///     `NSTDGLIndexFormat format` - The index format of the buffer.
+///     `const NSTDGLRenderPass render_pass` - The render pass.
+///     `const NSTDGLBuffer buffer` - The GPU index buffer.
+///     `const NSTDGLIndexFormat format` - The index format of the buffer.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_gl_render_pass_set_index_buffer(
@@ -650,7 +721,7 @@ pub unsafe extern "C" fn nstd_gl_render_pass_set_index_buffer(
 
 /// Draws primitives from active vertex buffers.
 /// Parameters:
-///     `NSTDGLRenderPass render_pass` - The render pass.
+///     `const NSTDGLRenderPass render_pass` - The render pass.
 ///     `const NSTDUInt32 verticies` - Number of verticies to draw.
 ///     `const NSTDUInt32 instances` - Number of instnaces.
 #[inline]
@@ -665,7 +736,7 @@ pub unsafe extern "C" fn nstd_gl_render_pass_draw(
 
 /// Draws primitives from active vertex and index buffers.
 /// Parameters:
-///     `NSTDGLRenderPass render_pass` - The render pass.
+///     `const NSTDGLRenderPass render_pass` - The render pass.
 ///     `const NSTDUInt32 indicies` - The indicies to draw.
 ///     `const NSTDUInt32 instances` - The instances to draw.
 ///     `const NSTDInt32 base` - The base vertex.
@@ -686,14 +757,13 @@ pub unsafe extern "C" fn nstd_gl_render_pass_draw_indexed(
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_gl_device_info_free(device_info: &mut NSTDGLDeviceInfo) {
-    drop(CString::from_raw(device_info.name));
-    device_info.name = ptr::null_mut();
+    crate::string::nstd_string_free(&mut device_info.name);
 }
 
 /// Creates a new GPU buffer.
 /// Parameters:
 ///     `const NSTDSlice *const bytes` - The bytes to send to the GPU.
-///     `NSTDGLDevice device` - The device to create the buffer on.
+///     `const NSTDGLDevice device` - The device to create the buffer on.
 /// Returns: `NSTDGLBuffer buffer` - The new GPU buffer.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
@@ -717,5 +787,5 @@ pub unsafe extern "C" fn nstd_gl_buffer_new(
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_gl_buffer_free(buffer: &mut NSTDGLBuffer) {
     Box::from_raw(*buffer);
-    *buffer = ptr::null_mut();
+    *buffer = std::ptr::null_mut();
 }
